@@ -1,8 +1,8 @@
 import enum
 import typing
 import logging
+import dataclasses
 
-import attr
 import zigpy.zdo.types
 import zigpy_znp.types as t
 
@@ -10,17 +10,13 @@ import zigpy_znp.types as t
 LOGGER = logging.getLogger(__name__)
 
 
-@attr.s
 class BindEntry(t.Struct):
     """Bind table entry."""
 
-    Src = attr.ib(type=t.EUI64, converter=t.Struct.converter(t.EUI64))
-    SrcEp = attr.ib(type=t.uint8_t, converter=t.uint8_t)
-    ClusterId = attr.ib(type=t.ClusterId, converter=t.ClusterId)
-    DstAddr = attr.ib(
-        type=zigpy.zdo.types.MultiAddress,
-        converter=t.Struct.converter(zigpy.zdo.types.MultiAddress),
-    )
+    Src: t.EUI64
+    SrcEp: t.uint8_t
+    ClusterId: t.ClusterId
+    DstAddr: zigpy.zdo.types.MultiAddress
 
 
 class CommandType(t.enum_uint8):
@@ -160,16 +156,12 @@ class CommandHeader(t.uint16_t):
     __repr__ = __str__
 
 
-@attr.s
+@dataclasses.dataclass(frozen=True)
 class CommandDef:
-    command_type: CommandType = attr.ib()
-    command_id: int = attr.ib()
-    req_schema: typing.Optional[t.Schema] = attr.ib(
-        factory=lambda *v: None if not v else t.Schema(*v)
-    )
-    rsp_schema: typing.Optional[t.Schema] = attr.ib(
-        factory=lambda *v: None if not v else t.Schema(*v)
-    )
+    command_type: CommandType
+    command_id: t.uint8_t
+    req_schema: typing.Optional[tuple] = None
+    rsp_schema: typing.Optional[tuple] = None
 
 
 class CommandsMeta(type):
@@ -316,8 +308,8 @@ class CommandBase:
     def __init__(self, *, partial=False, **params):
         super().__setattr__("_partial", partial)
 
-        all_params = [p.name for p in self.schema.parameters]
-        optional_params = [p.name for p in self.schema.parameters if p.optional]
+        all_params = [p.name for p in self.schema]
+        optional_params = [p.name for p in self.schema if p.optional]
         given_params = set(params.keys())
         given_optional = [p for p in params.keys() if p in optional_params]
 
@@ -343,7 +335,7 @@ class CommandBase:
 
         bound_params = {}
 
-        for param in self.schema.parameters:
+        for param in self.schema:
             if params.get(param.name) is None and (partial or param.optional):
                 bound_params[param.name] = (param, None)
                 continue
@@ -414,27 +406,28 @@ class CommandBase:
         data = frame.data
         params = {}
 
-        for param in cls.schema.parameters:
+        for param in cls.schema:
             if data:
                 params[param.name], data = param.type.deserialize(data)
             elif not param.optional:
                 # If we're out of data but the parameter is required, this is bad
                 raise ValueError(
-                    f"Frame has been parsed but parameters remain: {param}"
+                    f"Data has been consumed but required parameters remain: {param}"
                 )
             else:
                 # If we're out of data and the parameter is optional, we're done
                 break
 
         if data:
+            msg = (
+                f"Unparsed data remains at the end of the frame while parsing {cls}"
+                f" (parsed {params}): {data!r}"
+            )
+
             if ignore_unparsed:
-                LOGGER.warning(
-                    f"Unparsed data remains in {cls} at the end of the frame: {data!r}"
-                )
+                LOGGER.warning(msg)
             else:
-                raise ValueError(
-                    f"Unparsed data remains in {cls} at the end of the frame: {data!r}"
-                )
+                raise ValueError(msg)
 
         return cls(**params)
 
@@ -458,6 +451,16 @@ class CommandBase:
 
         return True
 
+    def replace(self, **kwargs) -> "CommandBase":
+        """
+        Returns a copy of the current command with replaced parameters.
+        """
+
+        params = {key: value for key, (param, value) in self._bound_params.items()}
+        params.update(kwargs)
+
+        return type(self)(partial=self._partial, **params)
+
     def __eq__(self, other):
         return type(self) is type(other) and self._bound_params == other._bound_params
 
@@ -486,20 +489,41 @@ class CommandBase:
     __str__ = __repr__
 
 
-class DeviceState(t.MissingEnumMixin, t.enum_uint8):
+class DeviceState(t.enum_uint8):
     """Indicated device state."""
 
+    # Initialized - not started automatically
     InitializedNotStarted = 0x00
+    # Initialized - not connected to anything
     InitializedNotConnected = 0x01
+    # Discovering PAN's to join
     DiscoveringPANs = 0x02
+    # Joining a PAN
     Joining = 0x03
-    ReJoining = 0x04
+    # ReJoining a PAN in secure mode scanning in current channel, only for end devices
+    ReJoiningSecureScanningCurrentChannel = 0x04
+    # Joined but not yet authenticated by trust center
     JoinedNotAuthenticated = 0x05
+    # Started as device after authentication
     JoinedAsEndDevice = 0x06
+    # Device joined, authenticated and is a router
     JoinedAsRouter = 0x07
+    # Started as Zigbee Coordinator
     StartingAsCoordinator = 0x08
+    # Started as Zigbee Coordinator
     StartedAsCoordinator = 0x09
+    # Device has lost information about its parent
     LostParent = 0x0A
+    # Device is sending KeepAlive message to its parent
+    SendingKeepAliveToParent = 0x0B
+    # Device is waiting before trying to rejoin
+    BackoffBeforeRejoin = 0x0C
+    # ReJoining a PAN in secure mode scanning in all channels, only for end devices
+    RejoinSecureScanningAllChannels = 0x0D
+    # ReJoining a PAN in unsecure mode scanning in current channel, only for end devices
+    RejoinInsecureScanningCurrentChannel = 0x0E
+    # ReJoining a PAN in unsecure mode scanning in all channels, only for end devices
+    RejoinInsecureScanningAllChannels = 0x0F
 
 
 class InterPanCommand(t.enum_uint8):
@@ -528,15 +552,14 @@ class MTCapabilities(t.enum_flag_uint16):
     CAP_UNK16 = 1 << 15
 
 
-@attr.s
 class Network(t.Struct):
-    PanId = attr.ib(type=t.PanId, converter=t.Struct.converter(t.PanId))
-    Channel = attr.ib(type=t.uint8_t, converter=t.uint8_t)
-    StackProfileVersion = attr.ib(type=t.uint8_t, converter=t.uint8_t)
-    BeaconOrderSuperframe = attr.ib(type=t.uint8_t, converter=t.uint8_t)
-    PermitJoining = attr.ib(type=t.uint8_t, converter=t.uint8_t)
+    PanId = t.PanId
+    Channel = t.uint8_t
+    StackProfileVersion = t.uint8_t
+    BeaconOrderSuperframe = t.uint8_t
+    PermitJoining = t.uint8_t
 
 
-STATUS_SCHEMA = t.Schema(
-    (t.Param("Status", t.Status, "Status is either Success (0) or Failure (1)"),)
+STATUS_SCHEMA = (
+    t.Param("Status", t.Status, "Status is either Success (0) or Failure (1)"),
 )
